extends MeshInstance3D

@export var navigation_region_np: NodePath
@export var range_begin := Vector3.ZERO
@export var range_end := Vector3.ONE
@export_range(1, 999999) var speed = 1
@export var optimize_path := false
@export var clear_each_time := false

var _rng = RandomNumberGenerator.new()
var _started = false

@onready var _lines = mesh
@onready var _timer = find_child("Timer")
@onready var _commit_timer = find_child("CommitTimer")


func _ready():
	_timer.timeout.connect(_on_timer_timeout)
	_timer.start(1 / float(speed))
	_commit_timer.timeout.connect(_on_commit_timer_timeout)
	_commit_timer.start(1.0)


func _draw_single_path():
	var begin = _get_random_navigation_point_in_range()
	var end = _get_random_navigation_point_in_range()
	var navigation_map_rid = get_world_3d().get_navigation_map()
	var path = NavigationServer3D.map_get_path(navigation_map_rid, begin, end, optimize_path)
	if path != null and path.size() > 1:
		if not _started:
			_lines.surface_begin(Mesh.PRIMITIVE_LINES)
			_started = true
		for point_id in range(path.size()):
			var point = path[point_id]
			_lines.surface_set_color(Color.BLACK)
			_lines.surface_set_normal(Vector3(0, 1, 0))
			_lines.surface_add_vertex(point)
			if point_id != 0 and point_id != path.size() - 1:
				_lines.surface_add_vertex(point)


func _get_random_navigation_point_in_range():
	return Vector3(
		_rng.randf_range(range_begin.x, range_end.x),
		range_begin.y,
		_rng.randf_range(range_begin.z, range_end.z)
	)


func _on_timer_timeout():
	if clear_each_time and _started:
		_lines.surface_end()
		_lines.clear_surfaces()
		_lines.surface_begin(Mesh.PRIMITIVE_LINES)
	_draw_single_path()


func _on_commit_timer_timeout():
	_lines.surface_end()
	_lines.surface_begin(Mesh.PRIMITIVE_LINES)
